/*
 * QuickTime for Java SDK Sample Code
 * Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.
 */

import quicktime.std.movies.*;
import quicktime.*;
import quicktime.std.StdQTConstants;
import quicktime.qd.*;
import quicktime.util.*;

/* this class contains methods to add sprite shared images to the key
		frame atom container */
public class KeyFrame extends SpriteAtom
{
		/* the keyFrame atom container */
	private AtomContainer keyFrameSample;
	
		/* the kSpriteSharedDataAtomType atom */
	private Atom spriteSharedDataAtom;
	private static final int sharedDataAtomID = 1;
	
		/* the kSpriteImageContainerAtomType atom */
	private Atom spriteImageContainerAtom;
	private static final int imageContainerAtomID = 1;
	private static final int spriteImageDataAtomTypeID = 1;
	private static final int spriteImageRegistrationAtomTypeID = 1;
	private static final int spriteNameAtomTypeID = 1;
	private static final int spriteGroupIDAtomTypeID = 1;

	KeyFrame(AtomContainer container)
	{
		super(container);
		keyFrameSample = container;
		
		spriteSharedDataAtom = null;
		spriteImageContainerAtom = null;
	}
	
		/* add the kSpriteSharedDataAtomType and kSpriteImageContainerAtomType atoms
			for our shared image data */
	private void addSharedDataAtomsToKeyFrameSample()
	{
		try
		{
			 spriteSharedDataAtom = keyFrameSample.insertChild(
												new Atom(StdQTConstants.kParentAtomIsContainer),
												StdQTConstants.kSpriteSharedDataAtomType,
												sharedDataAtomID,
												0);
			if (spriteSharedDataAtom != null)
				 spriteImageContainerAtom = keyFrameSample.insertChild(
														spriteSharedDataAtom, 
														StdQTConstants.kSpriteImagesContainerAtomType, 
														imageContainerAtomID, 
														0);
		}
		catch (QTException qte)
		{
			qte.printStackTrace();
		}
	}
	
		/* add a kSpriteImageAtomType atom plus the associated
			kSpriteImageDataAtomType, kSpriteNameAtomType,
			kSpriteImageRegistrationAtomType and kSpriteImageGroupIDAtomType atoms
			for a sprite image */
	public void addSpriteImageToSpriteImageContainer(int imageID,
											int imageIndex,
											QTHandle imageHandle,
											String imageName,
											QDPoint regPt,
											int groupID)
	{
		if ((spriteSharedDataAtom == null) && (spriteImageContainerAtom == null))
			addSharedDataAtomsToKeyFrameSample();

		try
		{
				/* add the kSpriteImageAtomType atom */
			Atom spriteImageAtom =
				keyFrameSample.insertChild(spriteImageContainerAtom,
									StdQTConstants.kSpriteImageAtomType,
									imageID,
									imageIndex);
			if (spriteImageAtom != null)
			{
				if (imageHandle != null)
				{
						/* add the kSpriteImageDataAtomType atom, specifying
							a handle with the image */
					keyFrameSample.insertChild(spriteImageAtom, 
											StdQTConstants.kSpriteImageDataAtomType,
											spriteImageDataAtomTypeID,
											0,
							     				imageHandle);
				}
				
				if (imageName != null)
				{
						/* add the kSpriteNameAtomType atom, specifying
							a name to associate with the sprite image */
					keyFrameSample.insertChild(spriteImageAtom, 
											StdQTConstants.kSpriteNameAtomType,
											spriteNameAtomTypeID,
											0,
							     				imageName.getBytes());

				}
				
				if (regPt != null)
				{
					int x,y;
					QDPoint thePoint = regPt;
	
						/* let's first endian-flip our QDPoint */
					EndianDescriptor endianDescForFixedPt = thePoint.getEndianDescriptorFixedPoint();
					byte[] fixedPtAsArray = thePoint.getFixedPoint();
					EndianOrder.flipNativeToBigEndian(fixedPtAsArray,0,endianDescForFixedPt);
					
						/* add the kSpriteImageRegistrationAtomType atom,
							specifying the registration point for the sprite image */
					keyFrameSample.insertChild(spriteImageAtom, 
											StdQTConstants.kSpriteImageRegistrationAtomType,
											spriteImageRegistrationAtomTypeID,
											0,
							     				fixedPtAsArray);
				}
				
				if (groupID != 0)
				{
						/* add the kSpriteImageGroupIDAtomType atom,
							specifying the groupID value for the image */
					keyFrameSample.insertChild(spriteImageAtom, 
											StdQTConstants.kSpriteImageGroupIDAtomType,
											spriteGroupIDAtomTypeID,
											0,
							     				EndianOrder.flipNativeToBigEndian32(groupID));
				}
			}
		}
		catch (QTException qte)
		{
			qte.printStackTrace();
		}
	}
}